(* 
 * Copyright (c) 2001 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: ksi.mli,v 1.3 2002/02/07 15:01:53 malekith Exp $
 *
 * Module interface: Ksi
 * Description: Interface to the Ksi intermediate language.
 *)

type id

(* Ksi typesystem. *)
type ty =
          Void
        | Bool
        | Int of int            (* int is size *)
        | UInt of int
        | Float of int
        | Ptr of ty             (* ptr to ... *)
        | FuncPtr of ty * (ty list)
                                (* return_type * parameters *)
        | Elipsis               (* `...' to be used only in function parms *)
        | Struct of id
        | Enum of id
        | Union of id
;;

val make_id : string -> id      (* given a string, return Ksi identifier. *)

val ptroff : unit -> ty         (* signed integer of the same size as native
                                 * pointer *)
val uptroff : unit -> ty        (* ditto, but unsigned *)
;;

(* C language types *)

type c_type_type =
(* size_of(c_type(C_{float,{long_,}double})) is only valid size for floating
 * point type. *)
          C_float
        | C_double
        | C_long_double

(* C integer types (are they needed for anything at all?) *)
        | C_char
        | C_short
        | C_int
        | C_long
        | C_long_long
        | C_uchar
        | C_ushort
        | C_uint
        | C_ulong
        | C_ulong_long
(* C pointer type, Ptr(Void) actually *)
        | C_ptr

val c_type : c_type_type -> ty

(* return size of object in bits. *)
val size_of : ty -> int

(* Attributes that associated with functions, variables and types. 
 * At the very moment only Extern, Local and Public are used. However I
 * hope to use NoReturn and MallocLike some day... *)
type attr =
          Aligned of int
        | Packed
        | Local         (* this is the default anyway... *)
        | Public
        | Extern
        | SuppressDebug
        | ConstFunc
        | PureFunc
        | StaticCtor
        | StaticDtor
        | MallocLike
        | NoReturn
        | NoLimitStack
        | NoCheckMemoryUsage
        | Artificial
        | Weak
        | OneOnly
        | Comdat
        | Section of string
;;

type binop =
          Eq            (* == *)
        | Ne            (* != *)
        | Ge            (* >= *)
        | Gt            (* > *)
        | Le            (* <= *)
        | Lt            (* < *)
        | Plus          (* + *)
        | Minus         (* - *)
        | Mult          (* * *)
        | RDiv          (* float / float *)
        | ExactDiv      (* division to use when no modulo is expected, like
                         * division of pointer diffrence by object size 
                         * in C. *)
        | RoundDiv      (* round result *)
        | FloorDiv      (* round result down *)
        | CeilDiv       (* round result up *)
        | TruncDiv      (* truncate result *)
        (* corresponding modulus operations *)
        | RoundMod | FloorMod | CeilMod | TruncMod
        
        | BitAnd        (* & *)
        | BitIOr        (* | *)
        | BitAndTc      (* sth strange *)
        | BitXor        (* ^ *)
        | Max
        | Min
        | LRotate
        | LShift        (* << *)
        | RRotate
        | RShift        (* >> *)
        (* ++/--, first operand is l-value to modify,
         * second operand is how much to inc/decremnt by. *)
        | PostInc | PreInc 
        | PostDec | PreDec
        | TruthAndIf    (* && *)
        | TruthOrIf     (* || *)
        | TruthAnd      (* similar to && but not lazy *)
        | TruthOr       (* similar to || but not lazy *)
        | TruthXor      (* same, but ^ is used instead of | or & *)
;;

type unop =
          Negate                (* -X *)
        | BitNot                (* ~X *)
        | TruthNot              (* !X *)
        | Abs
        | Addr                  (* &X *)
        | Ffs                   (* position of first bit set *)
        | NonLvalue             (* Hm... this is strange... *)
;;

type exp =
    (* (const ...) term *)
      IntConst of ty * string
      (* string is decimal representation used to avoid problems with
       * diffrent size of OCaml's int, int32 or int64 and target integer
       * size. It is Ksi resposiblity to convert this correctly. *)
    | FloatConst of ty * string             (* Ditto. *)
    | BoolConst of bool                     (* There are no such problems with
                                             * bool... :^) *)
    | StringConst of string                 (* There might be with string
                                             * (unicode), but for now... *)
    | Null of ty                            (* null constant of type ty *)
    (* Single term of similar name: *)
    | Var of ty * id
    | StaticVar of ty * id
    | Bind of exp           (* create scope *)
    | Ref of id             (* reference variable by name *)
    | FuncRef of id         (* get address of named function *)
    | Modify of exp * exp   (* exp = exp *)
    | Component of exp * id (* exp.string *)
    | Arrow of exp * id     (* exp->string, shortcut *)
    (* this is shortcut to (return (modify (result) exp))
     * exp can be Nop, (return (nil)) is generated then *)
    | Return of exp
    | SizeOf of ty                  (* Returns (const .sizetype ...). *)
    | Compound of exp * exp         (* exp, exp *)
    | CompoundList of exp list      (* exp1, exp2, ..., expn *)
    | Label of id
    | Goto of id
    | Loop of exp                   (* while (1) exp; *)
    | ExitLoop of (* bool *) exp    (* if (exp) break; *)
    | Nop
    | Indirect of exp               (* *exp *)
    | Cond of (* bool *) exp * exp * exp    (* exp ? exp : exp *)
    | Call of id * exp list             (* call named function, passing
                                         * paramatrers *)
    | FCall of id * exp list            (* call function that is
                                         * guaranted not to have 
                                         * any side-effects *)
    | PtrCall of exp * exp list
    | PtrFCall of exp * exp list
    | BuiltIn of id * exp list
    | FBuiltIn of id * exp list
    | Convert of ty * exp
    | Block of id * exp                 (* create named block *)
    | Exit of id                        (* jump to end of named block *)
    | Binary of binop * exp * exp
    | Unary of unop * exp
    | Line of int * exp                     (* annotate expression with
                                             * linenumber information. *)
    | Comment of string * exp               (* annotate output Ksi code
                                             * with comment *)
;;

        
type decl =
          FuncProto of (attr list) * ty * id * (ty list)
        | FuncProtoNamed of (attr list) * ty * id * ((ty * id) list)
        | Func of (attr list) * ty * id * (ty * id) list * exp
        | TopComment of string          (* output comment *)
        | StructDefProto of id
        | StructDef of id * attr list * (ty * id) list
        | UnionDefProto of id
        | UnionDef of id * attr list * (ty * id) list
        | Global of attr list * ty * id
        | InputFile of string
;;

(* Assertitions about the target. *)
val target_bytes_bigendian : unit -> bool
val target_words_bigendian : unit -> bool
val ksi_version : unit -> int

(* Read configuration file(s) from given directory. *)
val read_target_conf : (* prefix *) string -> unit

(* Print header of Ksi file to given channel. *)
val output_header : out_channel -> unit

(* Print one declaration to output channel. *)
val output_decl : out_channel -> decl -> unit

(* Finish writing Ksi file. *)
val output_trailer : out_channel -> unit

(* Raised if there is problem reading configuration file(s). *)
exception Conf_error of string

(* Raised in some undefined cases... ;) *)
exception Sorry of string

(* Set to true to get ugly-looking Ksi code (generation of it is however 
 * much faster). *)
val fast_writer : bool ref
;;
